@@ -22,6 +22,9 @@
 #' \code{mediator.type = 'gaussian'}. If \code{TRUE}, specify the desired quantile(s) using the \code{tau} parameter; otherwise, the default \code{tau = 0.5} (i.e., median) is used.
 #' @param efficient logical. Indicates whether to use efficient HIMA (\code{hima_efficient}). Default is \code{FALSE}. Applicable only for classic HIMA with a continuous outcome and 
 #' \code{mediator.type = 'gaussian'}.
+#' @param longitudinal logical. Indicates whether to run the longitudinal survival mediation model \code{hima_survival_long} (requires a \code{Surv(tstart, tstop, status)} outcome). 
+#' Default = \code{FALSE}.
+#' @param id.var Character string specifying the column name in \code{data.pheno} that stores subject identifiers. Required when \code{longitudinal = TRUE}.
 #' @param scale logical. Determines whether the function scales the data (exposure, mediators, and covariates). Default is \code{TRUE}. Note: For simulation studies, set 
 #' \code{scale = FALSE} to avoid estimate compression (i.e., shrinkage of estimates toward zero due to scaling).
 #' @param sigcut numeric. The significance cutoff for selecting mediators. Default is \code{0.05}.
@@ -44,27 +47,32 @@
 #' }
 #'
 #' @references
-#' 1. Zhang H, Zheng Y, Zhang Z, Gao T, Joyce B, Yoon G, Zhang W, Schwartz J, Just A, Colicino E, Vokonas P, Zhao L,
+#' 1. Zhang H, Zheng Y, Hou L, Liu L, HIMA: An R Package for High-Dimensional Mediation Analysis. Journal of Data Science. 2025. DOI: 10.6339/25-JDS1192
+#' 
+#' 2. Zhang H, Zheng Y, Zhang Z, Gao T, Joyce B, Yoon G, Zhang W, Schwartz J, Just A, Colicino E, Vokonas P, Zhao L,
 #' Lv J, Baccarelli A, Hou L, Liu L. Estimating and Testing High-dimensional Mediation Effects in Epigenetic Studies.
 #' Bioinformatics. 2016. DOI: 10.1093/bioinformatics/btw351. PMID: 27357171; PMCID: PMC5048064
-#'
-#' 2. Zhang H, Zheng Y, Hou L, Zheng C, Liu L. Mediation Analysis for Survival Data with High-Dimensional Mediators.
+#' 
+#' 3. Zhang H, Zheng Y, Hou L, Zheng C, Liu L. Mediation Analysis for Survival Data with High-Dimensional Mediators.
 #' Bioinformatics. 2021. DOI: 10.1093/bioinformatics/btab564. PMID: 34343267; PMCID: PMC8570823
-#'
-#' 3. Zhang H, Chen J, Feng Y, Wang C, Li H, Liu L. Mediation Effect Selection in High-dimensional and Compositional Microbiome data.
+#' 
+#' 4. Zhang H, Chen J, Feng Y, Wang C, Li H, Liu L. Mediation Effect Selection in High-dimensional and Compositional Microbiome data.
 #' Stat Med. 2021. DOI: 10.1002/sim.8808. PMID: 33205470; PMCID: PMC7855955
-#'
-#' 4. Zhang H, Chen J, Li Z, Liu L. Testing for Mediation Effect with Application to Human Microbiome Data.
+#' 
+#' 5. Zhang H, Chen J, Li Z, Liu L. Testing for Mediation Effect with Application to Human Microbiome Data.
 #' Stat Biosci. 2021. DOI: 10.1007/s12561-019-09253-3. PMID: 34093887; PMCID: PMC8177450
-#'
-#' 5. Perera C, Zhang H, Zheng Y, Hou L, Qu A, Zheng C, Xie K, Liu L. HIMA2: High-dimensional Mediation Analysis and Its Application in
+#' 
+#' 6. Perera C, Zhang H, Zheng Y, Hou L, Qu A, Zheng C, Xie K, Liu L. HIMA2: High-dimensional Mediation Analysis and Its Application in
 #' Epigenome-wide DNA Methylation Data. BMC Bioinformatics. 2022. DOI: 10.1186/s12859-022-04748-1. PMID: 35879655; PMCID: PMC9310002
-#'
-#' 6. Zhang H, Hong X, Zheng Y, Hou L, Zheng C, Wang X, Liu L. High-Dimensional Quantile Mediation Analysis with Application to a Birth
+#' 
+#' 7. Zhang H, Hong X, Zheng Y, Hou L, Zheng C, Wang X, Liu L. High-Dimensional Quantile Mediation Analysis with Application to a Birth
 #' Cohort Study of Mother–Newborn Pairs. Bioinformatics. 2024. DOI: 10.1093/bioinformatics/btae055. PMID: 38290773; PMCID: PMC10873903
-#'
-#' 7. Bai X, Zheng Y, Hou L, Zheng C, Liu L, Zhang H. An Efficient Testing Procedure for High-dimensional Mediators with FDR Control.
+#' 
+#' 8. Bai X, Zheng Y, Hou L, Zheng C, Liu L, Zhang H. An Efficient Testing Procedure for High-dimensional Mediators with FDR Control.
 #' Statistics in Biosciences. 2024. DOI: 10.1007/s12561-024-09447-4.
+#' 
+#' 9. Liu L, Zhang H, Zheng Y, Gao T, Zheng C, Zhang K, Hou L, Liu L. High-dimensional mediation analysis for longitudinal mediators and 
+#' survival outcomes. Briefings in Bioinformatics. 2025. DOI: 10.1093/bib/bbaf206. PMID: 40350699 PMCID: PMC12066418
 #'
 #' @examples
 #' \dontrun{
@@ -80,8 +88,9 @@
 #'   data.M = mediator_data,
 #'   mediator.type = "gaussian",
 #'   penalty = "MCP", # Can be "DBlasso" for hima_dblasso
-#'   scale = FALSE
-#' ) # Disabled only for simulation data
+#'   scale = FALSE, # Disabled only for simulation data
+#'   verbose = TRUE
+#' ) 
 #' summary(e1)
 #'
 #' # Efficient HIMA (only applicable to mediators and outcomes that are
@@ -92,8 +101,9 @@
 #'   mediator.type = "gaussian",
 #'   efficient = TRUE,
 #'   penalty = "MCP", # Efficient HIMA does not support DBlasso
-#'   scale = FALSE
-#' ) # Disabled only for simulation data
+#'   scale = FALSE, # Disabled only for simulation data
+#'   verbose = TRUE
+#' ) 
 #' summary(e1e)
 #'
 #' # Example 2 (binary outcome - logistic HIMA):
@@ -106,8 +116,9 @@
 #'   data.M = mediator_data,
 #'   mediator.type = "gaussian",
 #'   penalty = "MCP",
-#'   scale = FALSE
-#' ) # Disabled only for simulation data
+#'   scale = FALSE, # Disabled only for simulation data
+#'   verbose = TRUE
+#' ) 
 #' summary(e2)
 #'
 #' # Example 3 (time-to-event outcome - survival HIMA):
@@ -120,10 +131,28 @@
 #'   data.M = mediator_data,
 #'   mediator.type = "gaussian",
 #'   penalty = "DBlasso",
-#'   scale = FALSE
-#' ) # Disabled only for simulation data
+#'   scale = FALSE, # Disabled only for simulation data
+#'   verbose = TRUE
+#' ) # Parallel computing feature is recommended
 #' summary(e3)
+#' 
+#' # Longitudinal mediator + survival HIMA:
+#' data(SurvivalLongData)
+#' pheno_data <- SurvivalLongData$PhenoData
+#' mediator_data <- SurvivalLongData$Mediator
 #'
+#' e3long <- hima(Surv(Tstart, Tstop, Status) ~ Treatment + Sex + Age,
+#'   data.pheno = pheno_data,
+#'   data.M = mediator_data,
+#'   mediator.type = "gaussian",
+#'   penalty = "lasso",
+#'   longitudinal = TRUE,
+#'   id.var = "ID",
+#'   scale = FALSE, # Disabled only for simulation data
+#'   verbose = TRUE
+#' ) # Parallel computing feature is recommended
+#' summary(e3long)
+#' 
 #' # Example 4 (compositional data as mediator, e.g., microbiome):
 #' data(MicrobiomeData)
 #' pheno_data <- MicrobiomeData$PhenoData
@@ -133,11 +162,12 @@
 #'   data.pheno = pheno_data,
 #'   data.M = mediator_data,
 #'   mediator.type = "compositional",
-#'   penalty = "DBlasso"
+#'   penalty = "DBlasso",
+#'   verbose = TRUE
 #' ) # Scaling is always enabled internally for hima_microbiome
 #' summary(e4)
 #'
-#' #' # Example 5 (quantile mediation anlaysis - quantile HIMA):
+#' #' # Example 5 (quantile mediation analysis - quantile HIMA):
 #' data(QuantileData)
 #' pheno_data <- QuantileData$PhenoData
 #' mediator_data <- QuantileData$Mediator
@@ -150,7 +180,8 @@
 #'   quantile = TRUE,
 #'   penalty = "MCP", # Quantile HIMA does not support DBlasso
 #'   scale = FALSE, # Disabled only for simulation data
-#'   tau = c(0.3, 0.5, 0.7)
+#'   tau = c(0.3, 0.5, 0.7),
+#'   verbose = TRUE
 #' ) # Specify multiple quantile level
 #' summary(e5)
 #' }
@@ -163,6 +194,8 @@ hima <- function(formula,
                  penalty = c("DBlasso", "MCP", "SCAD", "lasso"),
                  quantile = FALSE,
                  efficient = FALSE,
+                 longitudinal = FALSE,
+                 id.var = NULL,
                  scale = TRUE,
                  sigcut = 0.05,
                  contrast = NULL,
@@ -211,29 +244,62 @@ hima <- function(formula,
   
   # extract data from formula and input dataset
   d <- .f_extract(data.pheno = data.pheno, f = formula)
-
+  
+  # check mutually exclusive analysis modes
+  mode_flags <- c(
+    efficient   = isTRUE(efficient),
+    quantile    = isTRUE(quantile),
+    longitudinal = isTRUE(longitudinal)
+  )
+  if (sum(mode_flags) > 1L) {
+    stop("Arguments 'efficient', 'quantile', and 'longitudinal' are mutually exclusive; please set at most one of them to TRUE.")
+  }
+  
   if (d$type == "categorical") stop("Currently HIMA does not support categorical outcome.")
-
-  if (efficient && quantile) stop("Cannot have both 'efficient' and 'quantile' set to TRUE simultaneously.")
-
+  
+  if (longitudinal && d$type != "survival") {
+    stop("Longitudinal mediation is only available for survival outcomes specified via Surv().")
+  }
+  
+  id_vec <- NULL
+  
+  if (longitudinal) {
+    if (is.null(d$Y$tstart) || is.null(d$Y$tstop)) {
+      stop("Please specify the survival outcome as Surv(tstart, tstop, status) when longitudinal = TRUE.")
+    }
+    if (is.null(id.var)) {
+      stop("Argument 'id.var' must be provided when longitudinal = TRUE.")
+    }
+    if (!id.var %in% names(data.pheno)) {
+      stop("The column specified in 'id.var' does not exist in data.pheno.")
+    }
+    id_vec <- data.pheno[[id.var]]
+  }
+  
   mediator.type <- match.arg(mediator.type)
   penalty <- match.arg(penalty)
-
+  
+  # Longitudinal: penalty must be lasso
+  if (longitudinal && penalty != "lasso") {
+    if (verbose) message("Note: For longitudinal survival mediation, only 'lasso' penalty is supported. \nSwitching to 'lasso' ...")
+    penalty <- "lasso"
+  }
+  
   if (sigcut > 1) sigcut <- 1
-
+  
   if (is.null(colnames(data.M))) colnames(data.M) <- seq_len(ncol(data.M))
-
+  
   # Conditions where 'DBlasso' is not applicable
-  invalid_dblasso <- efficient || quantile || d$type == "binary" || mediator.type == "negbin"
-
+  invalid_dblasso <- efficient || quantile || longitudinal || d$type == "binary" || mediator.type == "negbin"
+  
   # Conditions where 'DBlasso' must be used
-  require_dblasso <- d$type == "survival" || mediator.type == "compositional"
-
+  require_dblasso <- (!longitudinal) && (d$type == "survival" || mediator.type == "compositional")
+  
   # Check for conflicting conditions
   if (invalid_dblasso && require_dblasso) {
-    stop("Conflicting conditions: Cannot determine appropriate penalty under the \ncurrent settings.")
+    stop("Conflicting conditions: Cannot determine appropriate penalty under the current settings.")
   }
-
+  
   # Adjust 'penalty' based on the conditions
   if (penalty == "DBlasso" && invalid_dblasso) {
     if (verbose) message("Note: The selected conditions do not support de-biased Lasso penalty. \nSwitching to 'MCP' ...")
@@ -244,18 +310,17 @@ hima <- function(formula,
   }
   
   ######
-
-  # hima_efficient / hima_quantile
-  if (efficient || quantile) {
+  
+  # hima_efficient / hima_quantile / hima_survival_long
+  if (efficient || quantile || longitudinal) {
     if (efficient) {
       if (penalty != "MCP") {
         if (verbose) message("Note: Efficient HIMA works best with 'MCP' penalty. Switching penalty to 'MCP'...")
       }
       if (verbose) message("Running efficient HIMA with 'MCP' penalty...")
       if (d$type != "continuous" || mediator.type != "gaussian") {
-        stop("Efficient HIMA is only applicable to mediators and outcomes that are BOTH \ncontinuous and normally distributed.")
+        stop("Efficient HIMA is only applicable to mediators and outcomes that are BOTH continuous and normally distributed.")
       }
-
       res <- hima_efficient(
         X = d$X,
         M = data.M,
@@ -268,18 +333,14 @@ hima <- function(formula,
         parallel = parallel,
         ncore = ncore
       )
-
       results <- .res_prep(res, method_text = "DACT method with BH-FDR", sigcut = sigcut)
     }
-
+    
     if (quantile) {
       if (verbose) message("Running quantile HIMA with ", penalty, " penalty...")
       if (d$type != "continuous" || mediator.type != "gaussian") {
-        stop("Quantile HIMA is only applicable to mediators and outcomes that are BOTH \ncontinuous and normally distributed.")
+        stop("Quantile HIMA is only applicable to mediators and outcomes that are BOTH continuous and normally distributed.")
       }
-
-      # tau <- readline(prompt = "Enter quantile level(s) (between 0-1, multiple values accepted): ")
-      # tau <- eval(parse(text = paste0("c(", tau, ")")))
       res <- hima_quantile(
         X = d$X,
         M = data.M,
@@ -288,17 +349,36 @@ hima <- function(formula,
         penalty = penalty,
         topN = NULL,
         scale = scale,
-        Bonfcut = 0.05,
+        Bonfcut = sigcut,
         verbose = verbose,
         parallel = parallel,
         ncore = ncore,
         ...
       )
-
       results <- .res_prep(res, method_text = "Bonferroni-adjusted p", sigcut = sigcut, q = TRUE)
     }
-  } else { # If not hima_efficient AND not hima_quantile
-    # DBlasso
+    
+    if (longitudinal) {
+      if (verbose) message("Running longitudinal survival HIMA with L1-penalized Cox lasso...")
+      res <- hima_survival_long(
+        X = d$X,
+        M = data.M,
+        tstart = d$Y$tstart,
+        tstop = d$Y$tstop,
+        status = d$Y$status,
+        id = id_vec,
+        COV = d$COV,
+        topN = NULL,
+        scale = scale,
+        Bonfcut = sigcut,
+        verbose = verbose,
+        parallel = parallel,
+        ncore = ncore
+      )
+      results <- .res_prep(res, method_text = "Bonferroni-adjusted p", sigcut = sigcut)
+    }
+  } else { # If not hima_efficient AND not hima_quantile AND not hima_survival_long
+    
     if (penalty == "DBlasso") {
       if (d$type == "continuous") {
         if (mediator.type == "gaussian") {
@@ -315,7 +395,6 @@ hima <- function(formula,
             parallel = parallel,
             ncore = ncore
           )
-
           results <- .res_prep(res, method_text = "HDMT pointwise FDR", sigcut = sigcut)
         } else if (mediator.type == "compositional") {
           if (verbose) message("Running compositional HIMA with de-biased Lasso penalty...")
@@ -329,7 +408,6 @@ hima <- function(formula,
             parallel = parallel,
             ncore = ncore
           )
-
           results <- .res_prep(res, method_text = "Hommel FDR", sigcut = sigcut)
         }
       } else if (d$type == "survival") {
@@ -347,9 +425,8 @@ hima <- function(formula,
           parallel = parallel,
           ncore = ncore
         )
-
-        results <- .res_prep(res, method_text = "HDMT pointwise FDR", sigcut = sigcut)
       }
+      results <- .res_prep(res, method_text = "HDMT pointwise FDR", sigcut = sigcut)
     } else { # If penalty is not DBlasso
       res <- hima_classic(
         X = d$X,
@@ -366,13 +443,12 @@ hima <- function(formula,
         parallel = parallel,
         ncore = ncore
       )
-
       results <- .res_prep(res, method_text = "Bonferroni-adjusted p", sigcut = sigcut)
     }
   }
-
+  
   if (is.null(results) && verbose) message("No significant mediator found!")
-
+  
   return(structure(results, class = "hima"))
 }
 
@@ -468,8 +544,8 @@ summary.hima <- function(object, desc = FALSE, ...) {
   )
   if (q) {
     results <- data.frame(results,
-      tau = res$tau,
-      check.names = FALSE
+                          tau = res$tau,
+                          check.names = FALSE
     )
     attr_test <- c(
       attr_test,
@@ -487,7 +563,7 @@ summary.hima <- function(object, desc = FALSE, ...) {
 .convert_to_dummies <- function(df) {
   char_cols <- sapply(df, is.character)
   df[char_cols] <- lapply(df[char_cols], as.factor)
-  df_dummies <- as.data.frame(model.matrix(~ . - 1, data = df))
+  df_dummies <- as.data.frame(model.matrix(~ . - 1, data = df, na.action = na.pass))
   non_char_cols <- names(df)[!char_cols]
   df_dummies[non_char_cols] <- df[non_char_cols]
   return(df_dummies)
@@ -502,17 +578,17 @@ summary.hima <- function(object, desc = FALSE, ...) {
   # Get unique values
   uniq_vals <- unique(x_clean)
   num_unique <- length(uniq_vals)
-
+  
   # Handle variables with no data
   if (num_unique == 0) {
     stop("No valid outcome data detected!") # Variable contains only NA values
   }
-
+  
   # Classify binary variables
   if (num_unique == 1 || num_unique == 2) {
     return("binary")
   }
-
+  
   # Classify continuous and categorical variables
   if (is.numeric(x) || is.integer(x)) {
     if (num_unique > categorical_threshold) {
@@ -533,29 +609,49 @@ summary.hima <- function(object, desc = FALSE, ...) {
 
 ### Internal function: Formula extractor function
 .f_extract <- function(data.pheno, f) {
-  if (as.character(f[[2]])[1] == "Surv") {
-    response_vars <- as.character(f[[2]])[c(2, 3)]
-    ind_vars <- all.vars(f)[-c(1, 2)]
-    Y <- list(
-      OT = data.pheno[, response_vars[1]],
-      status = data.pheno[, response_vars[2]]
-    )
+  is_surv <- identical(as.character(f[[2]])[1], "Surv")
+  
+  if (is_surv) {
+    response_vars <- all.vars(f[[2]])
+    n_resp <- length(response_vars)
+    if (!n_resp %in% c(2, 3)) {
+      stop("Surv() must be specified as Surv(time, status) or Surv(tstart, tstop, status).")
+    }
+    
+    if (n_resp == 2) {
+      Y <- list(
+        OT = data.pheno[[response_vars[1]]],
+        status = data.pheno[[response_vars[2]]],
+        tstart = NULL,
+        tstop = NULL
+      )
+    } else {
+      Y <- list(
+        status = data.pheno[[response_vars[3]]],
+        tstart = data.pheno[[response_vars[1]]],
+        tstop = data.pheno[[response_vars[2]]]
+      )
+    }
+    ind_vars <- all.vars(f[[3]])
     type <- "survival"
   } else {
     response_var <- as.character(f[[2]])
-    ind_vars <- all.vars(f)[-1]
-    Y <- data.pheno[, response_var]
+    ind_vars <- all.vars(f[[3]])
+    Y <- data.pheno[[response_var]]
     type <- .data_type(Y)
   }
-
-  X <- data.pheno[, ind_vars[1]]
-
+  if (!length(ind_vars)) {
+    stop("No exposure/covariate specified on the right-hand side of the formula.")
+  }
+  
+  X <- data.pheno[[ind_vars[1]]]
+  
   if (length(ind_vars) > 1) {
     COV <- .convert_to_dummies(data.pheno[, ind_vars[-1], drop = FALSE])
   } else {
     COV <- NULL
   }
-
+  
   return(list(
     Y = Y,
     X = X,
@@ -579,26 +675,26 @@ hima2 <- function(formula,
                   verbose = FALSE,
                   ...) {
   .Deprecated(msg = "hima2() is deprecated. Please use hima() with updated parameters.")
-
+  
   outcome.family <- match.arg(outcome.family)
   if (outcome.family == "quantile") quantile <- TRUE else quantile <- FALSE
   if (!missing(outcome.family)) {
-    warning("'outcome.family' is deprecated. The outcome data type can now be \nautomatically recognized.")
+    warning("'outcome.family' is deprecated. The outcome data type can now be automatically recognized.")
   }
-
+  
   mediator.type <- match.arg(mediator.family)
   penalty <- match.arg(penalty)
-
+  
   hima(formula,
-    data.pheno,
-    data.M,
-    mediator.type = mediator.type,
-    penalty = penalty,
-    quantile = quantile,
-    efficient = efficient,
-    scale = scale,
-    sigcut = sigcut,
-    verbose = verbose,
-    ...
+       data.pheno,
+       data.M,
+       mediator.type = mediator.type,
+       penalty = penalty,
+       quantile = quantile,
+       efficient = efficient,
+       scale = scale,
+       sigcut = sigcut,
+       verbose = verbose,
+       ...
   )
 }
