@@ -61,6 +61,17 @@ hima_microbiome <- function(X,
                             verbose = FALSE,
                             parallel = FALSE,
                             ncore = 1) {
+  
+  ## Outcome checks: only continuous Y is allowed for now
+  if (is.factor(Y) && nlevels(Y) == 2L ||
+      is.numeric(Y) && all(stats::na.omit(Y) %in% c(0, 1))) {
+    stop("hima_microbiome() only supports continuous outcomes. ")
+  }
+  
+  if (!is.numeric(Y)) {
+    stop("Y must be numeric for hima_microbiome().")
+  }
+  
   X <- matrix(X, ncol = 1)
   
   M_raw <- as.matrix(OTU)
@@ -80,6 +91,11 @@ hima_microbiome <- function(X,
   M <- M_raw
   n <- dim(M)[1]
   d <- dim(M)[2]
+  
+  if (nrow(X) != n || length(Y) != n || (!is.null(COV) && nrow(COV) != n)) {
+    stop("X, OTU, Y, and COV (if provided) must have the same number of observations.")
+  }
+  
   alpha_EST <- matrix(0, 1, d)
   alpha_SE <- matrix(0, 1, d)
   beta_EST <- matrix(0, 1, d)
@@ -87,10 +103,10 @@ hima_microbiome <- function(X,
   P_raw_DLASSO <- matrix(0, 1, d)
   M1 <- t(t(M_raw[, 1]))
 
-  if (verbose) message("Step 1: ILR Transformation and De-biased Lasso estimates ...", "  (", format(Sys.time(), "%X"), ")")
-
   checkParallel("hima_microbiome", parallel, ncore, verbose)
   
+  if (verbose) message("Step 1: ILR Transformation and De-biased Lasso estimates ...", "  (", format(Sys.time(), "%X"), ")")
+
   if (verbose) {
     if (is.null(COV)) {
       message("        No covariate was adjusted.")
@@ -138,12 +154,12 @@ hima_microbiome <- function(X,
   
   P_adj_DLASSO <- as.numeric(P_raw_DLASSO)
   
-  if (verbose) message("Step 2: Closted testing-based procedure ...", "     (", format(Sys.time(), "%X"), ")")
+  if (verbose) message("Step 2: Closed testing-based procedure ...", "     (", format(Sys.time(), "%X"), ")")
   
   ## The FDR method
   set <- which(P_adj_DLASSO < FDRcut)
   hom <- hommel::hommel(P_adj_DLASSO, simes = FALSE)
-  N1 <- hommel::discoveries(hom, set, incremental = TRUE, alpha = 0.05)
+  N1 <- hommel::discoveries(hom, set, incremental = TRUE, alpha = FDRcut)
   
   if (length(set) > 0) {
     L <- length(set)
